/** \file Display.h
    \brief Abstract display interface.
*/

/* Copyright (C) 2008-2009 Tomasz Ostrowski <tomasz.o.ostrowski at gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.

 * Miniscope is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DisplayH
#define DisplayH

#include <vector>
#include <memory>
#include <Graphics.hpp>	// needed for CreateBitmap()

/** \brief Abstact display interface class
 */
class Display
{
public:
	/** \brief Display mode
	*/
	enum E_DISPLAY_MODE
	{
		MODE_OSCILLOSCOPE = 0,  ///< plot versus time
		MODE_SPECTRUM,          ///< plot versus frequency
		MODE_RECORDER			///< plot versus time, slow changing signal recorder
	};

	/** \brief Required level of graph update
	 *
	 *  Drawing is time-consuming. Display object can hold partial graph drawings
	 *  (i.e. graph background + graph grid) and reuse them when they do not need
	 *  to be updated.
	 */
	enum E_UPDATE_LEVEL
	{
		UPDATE_ALL = 0,
		UPDATE_DATA,
		UPDATE_CURSOR,
		UPDATE_REPAINT
	};

	/** \brief Cursor identifiers
	*/
	enum E_CURSOR
	{
		CURSOR1,
		CURSOR2,
		CURSOR_LIMIT
	};

	/** \brief Special values for SetZoom function passed as X or Y offset value
	*/
	enum E_OFFSET_ACTION
	{
		OFFSET_LEAVE = -2,	///< leave current offset
		OFFSET_CLEAR = -1	///< clear offset
	};

	Display(void);

	virtual ~Display() {};
	/** \brief Data to plot
	*/
	std::vector<float> *data;
	/** \brief Set time between samples
	*/
	virtual void SetDiv(float fX);
	/** \brief Switch between plot modes (time graph/FFT/...)
	*/
	virtual void SetMode(const enum E_DISPLAY_MODE m) = 0;
	/** \brief Get type of current display mode
	*/
	virtual const enum E_DISPLAY_MODE GetMode(void) {
		return mode;
	}
	/** \brief Update displayed content.
	 *  \param level Used to specify parts of graph that must be updated.
	 */
	virtual void Update(enum E_UPDATE_LEVEL level) = 0;
	/** \brief Resize viewport (visible area of plot)
	*/
	virtual void ChangeSize(int w, int h) = 0;
	/** \brief Set expected range of input data to -range...+range
	*/
	virtual void SetRange(float range) {
		fRange = range;
	}
	/** \brief Set position of cursor #1 */
	virtual void SetCursor1(int x, int y) = 0;
	/** \brief Set position of cursor #2 */
	virtual void SetCursor2(int x, int y) = 0;
	/** \brief Set viewport offset and zoom
		\param iCenterX	requested center of viewport (i.e. zoom click coordinate) or enum E_OFFSET_ACTION
		\param iCenterY requested center of viewport (i.e. zoom click coordinate) or enum E_OFFSET_ACTION
		\param fZoomX	X-axis scale
		\param fZoomY	Y-axis scale
	*/
	virtual void SetZoom(int iCenterX, int iCenterY, float fZoomX, float fZoomY) = 0;
	/** \brief Set Y-axis offset
		\param fVal requested value at center position
	*/
	virtual void SetOffsetY(float fVal) = 0;
	/** \brief Move viewport. */
	virtual void Move(int iStartX, int iStartY, int iStopX, int iStopY) = 0;
	/** \brief Transform screen coordinates to physical values */
	virtual void PosToPhysical(int x, int y, float &fValX, float &fValY) = 0;
	/** \brief Get physical (voltage/time or freq/time) coordinates of cursor
		\return 0 on success
	*/
	virtual int GetCursorPhysical(enum E_CURSOR cursor, float &fValX, float &fValY) = 0;
	virtual void SetVisibility(enum E_DISPLAY_MODE mode, bool bGridX, bool bGridY, bool bGridLabels);

	/** \todo MarkTrigger
	*/

	/** \brief Create plot bitmap
	*/
	virtual std::auto_ptr<Graphics::TBitmap> CreateBitmap(void) = 0;

protected:
	float fRange;				///< vertical range (range is from -fRange to fRange)
	float fDivX;                ///< time difference between samples
	enum E_DISPLAY_MODE mode;   ///< current display mode type

	/** \brief State of plot elements (visibility, zoom, position) in specific mode
	*/
	struct S_STATE
	{
		bool bGridX;            ///< should horizontal grid be visible?
		bool bGridY;            ///< should vertical grid be visible?

		bool bGridLabels;		///< show value labels next to grid?

		float fZoomX;           ///< horizontal zoom value
		float fZoomY;           ///< vertical zoom value

		float fOffsetX;         ///< X position of plot on canvas
		float fOffsetY;	        ///< Y position of plot on canvas

		float fCursor1X;        ///< cursor 1 position X
		float fCursor1Y;        ///< cursor 1 position Y
		float fCursor2X;        ///< cursor 2 position X
		float fCursor2Y;        ///< cursor 2 position Y
	};
	struct S_STATE scope;       ///< display state for scope mode
	struct S_STATE spectrum;    ///< display state for spectrum analyzer state
    struct S_STATE recorder;	///< display state for slow changing signal recorder

    /** \brief Get current plot state/zoom/offset/visibility
    */
	inline struct S_STATE& State(void) {
		if (mode == MODE_OSCILLOSCOPE)
			return scope;
		else if (mode == MODE_RECORDER)
        	return recorder;
		else
			return spectrum;
	}
	/** \brief Get plot state for specified mode
	*/
	inline struct S_STATE& State(enum E_DISPLAY_MODE mode) {
		if (mode == MODE_OSCILLOSCOPE)
			return scope;
		else if (mode == MODE_RECORDER)
        	return recorder;
		else
			return spectrum;
	}
	/** \brief Get padded data count for FFT mode
	*/
	unsigned int NearestPowerOfTwo(unsigned int count);
};

#endif // DisplayH
